@@ -12,6 +12,9 @@ import cairo
 import datetime
 import shutil
 import sys
+import socket
+
+socket.setdefaulttimeout(10)
 
 supports_alpha = False
 
@@ -22,7 +25,7 @@ supports_alpha = False
 
 # constants. dbfile is the location of the csv
 # comiccache is the location of the images
-APP_VERSION = "0.3.2-2"
+APP_VERSION = "0.3.4-2"
 
 basedbdir = "/opt/comic-widget/db/"
 imagedir = "/opt/comic-widget/images/"
@@ -33,7 +36,9 @@ comics = {"xkcd":{"name":"xkcd","link":"http://xkcd.org/","start":666,"dbfile":d
 		"sinfest":{"name":"Sinfest","link":"http://sinfest.com/","start":3400,"dbfile":dbdir + "comicdb.sinfest.csv"},
 		"phd":{"name":"PHD Comics","link":"http://www.phdcomics.com/","start":1240,"dbfile":dbdir + "comicdb.phd.csv"},
 		"dilbert":{"name":"Dilbert","link":"http://dilbert.com/","start":"2009-01-01","dbfile":dbdir + "comicdb.dilbert.csv"},
-		"cyanide":{"name":"C and H","link":"http://explosm.com/","start":"1920","dbfile":dbdir + "comicdb.cyanide.csv"},
+		"cyanide":{"name":"Cyanide and Happiness","link":"http://explosm.com/","start":"1920","dbfile":dbdir + "comicdb.cyanide.csv"},
+		"wulff":{"name":"Wulffmorgenthaler","link":"http://wulffmorgenthaler.com/","start":"edd3411b-96ca-4d93-bd5f-0cf1deb67c8a","dbfile":dbdir + "comicdb.wulff.csv"},
+
 		}
 defaults = {'width':480,'height':230}
 #defaults = {'width':480,'height':240}
@@ -135,7 +140,12 @@ class ComicDb():
 					print "created dir"
 				except:
 					print "comic db creation failed on mkdir"
-			urllib.urlretrieve(dbrow['url'], filename)
+			try:
+
+				urllib.urlretrieve(dbrow['url'], filename)
+			except:
+				if os.path.isfile(filename):
+					os.remove(filename)
 			print "success\n"
 			return filename
 
@@ -192,8 +202,11 @@ class ComicDb():
 # fetch earlier
 	def fetch_earlier(self, comic, earliest):
 		print "fetch before, " + comic + " earliest"
-		if comic == "cyanide":
+		if comic == "cyanide" or comic == "wulff":
 			comicid = self.get_prev_id(comic,earliest)
+			if not comicid:
+				print "already at first comic"
+				return
 			print "got " + comicid + " as the one before current..."
 		elif len(earliest) == 10:
 			# date id.
@@ -222,8 +235,11 @@ class ComicDb():
 
 
 	def fetch_newer(self, comic, newest):
-		if comic == "cyanide":
+		if comic == "cyanide" or comic == "wulff":
 			comicid = self.get_next_id(comic,newest)
+			if not comicid:
+				print "already at last comic"
+				return
 		elif len(newest) == 10:
 			# date id.
 			dt = string.split(newest, "-")
@@ -335,6 +351,38 @@ class ComicDb():
 				else:
 					print "got next: " + splt[0]
 					return splt[0]	
+		if comic == 'wulff':
+			link = "http://wulffmorgenthaler.com/default.aspx?id=" + number
+			print "link: " + link
+			try:
+				f = urllib2.urlopen(link)
+				hcode = f.code
+			except:
+				hcode = 404
+			print "got hcode = " + str(hcode) + "\n"
+			if (hcode != 200):
+				return False
+			else:
+				s = f.read()
+				f.close()
+				# title:
+				splt = string.split(s, '<a href="/default.aspx?id=', 1)
+				if not splt[1][36:69] == '" id="ctl00_content_Strip1_aPrev"':
+					print "no previous found"
+					if not splt[1][36:69] == '" id="ctl00_content_Strip1_aNext"':
+						print "no next found!"
+						return False
+				else:
+					splt = string.split(splt[1], '<a href="/default.aspx?id=', 1)
+				if len(splt) < 2:
+					print "no next found! At newest comic!"
+					return False
+				if not splt[1][36:69] == '" id="ctl00_content_Strip1_aNext"':
+					print "no next found!"
+					return False
+				else:
+					return splt[1][:36]
+
 
 	def get_prev_id(self, comic, number):
 		if comic == 'cyanide':
@@ -364,6 +412,30 @@ class ComicDb():
 				else:
 					print "got previous: " + splt[0]
 					return splt[0]				
+		if comic == 'wulff':
+			link = "http://wulffmorgenthaler.com/default.aspx?id=" + number
+			print "link: " + link
+			try:
+				f = urllib2.urlopen(link)
+				hcode = f.code
+			except:
+				hcode = 404
+			print "got hcode = " + str(hcode) + "\n"
+			if (hcode != 200):
+				return False
+			else:
+				s = f.read()
+				f.close()
+				# title:
+				splt = string.split(s, '<a href="/default.aspx?id=', 1)
+				if not splt[1][36:69] == '" id="ctl00_content_Strip1_aPrev"':
+					print splt[1][36:69]
+					print splt[1][:36]
+					print "no previous found"
+					exit()
+					return False
+				else:
+					return splt[1][:36]
 
 
 
@@ -387,6 +459,8 @@ class ComicDb():
 			link = "http://www.phdcomics.com/comics/archive.php?comicid=" + str(number)
 		elif comic == 'cyanide':
 			link = "http://www.explosm.net/comics/" + str(number) + "/"
+		elif comic == 'wulff':
+			link = "http://wulffmorgenthaler.com/default.aspx?id=" + str(number)
 		else:
 			return False
 
@@ -498,12 +572,43 @@ class ComicDb():
 				filename = splt2[1]
 				irow = [comic,number,link,url,filename,title,next,prev]
 				return irow
-	
+			elif comic == 'wulff':
+				# wulf is type .gif
+				s = f.read()
+				f.close()
+# <img src="pics/dates/month/2.gif" alt="" />
+# <img src="pics/dates/day/4.gif" alt="" />
+# <img src="pics/dates/komma.gif" alt="" />
+# <img src="pics/dates/year/2010.gif" alt="" />
+				splt = string.split(s, '<img src="pics/dates/month/', 1)
+				if not splt[1][1:2] == ".":
+					title = splt[1][:2]
+				else:
+					title = "0" + splt[1][:1]
+				splt = string.split(splt[1], '<img src="pics/dates/day/', 1)
+				if not splt[1][1:2] == ".":
+					title = title + "-" + splt[1][:2]
+				else:
+					title = title + "-0" + splt[1][:1]
+				splt = string.split(splt[1], '<img src="pics/dates/year/', 1)
+				title = splt[1][:4] + "-" + title
+				# title done...
+				splt = string.split(splt[1], 'id="ctl00_content_Strip1_imgStrip" class="strip" src="striphandler.ashx?stripid=', 1)
+				number = splt[1][:36]
+				url = "http://wulffmorgenthaler.com/striphandler.ashx?stripid=" + number
+				filename = number + ".gif"
+				irow = [comic,number,link,url,filename,title]
+				return irow
+				
+
 			splt2 = string.rsplit(url, "/", 1)
 			filename = splt2[1]
-			if filename == self.db[0]['filename']:
-				print "already exists! Break break break!"
-				return False
+			try:
+				if filename == self.db[0]['filename']:
+					print "already exists! Break break break!"
+					return False
+			except:
+				pass
 			irow = [comic,number,link,url,filename,title]
 			return irow
 
@@ -549,7 +654,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 		titl = str(self.db.db[self.db.currentcomic]['title'])
 		if len(titl) > 22:
 			titl = titl[:19] + "..."
-		self.label.set_markup('<span size="12000" face="monospace"> <b>' + self.comicname + '</b> ' + str(self.db.db[self.db.currentcomic]['id']) + '\n ' + titl + '</span>')
+		self.label.set_markup(self.get_markup())
 
 
 		self.e_goweb = gtk.EventBox()
@@ -732,7 +837,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 				titl = str(self.db.db[self.db.currentcomic]['title'])
 				if len(titl) > 22:
 					titl = titl[:19] + "..."
-				self.label.set_markup('<span size="12000" face="monospace"> <b>' + self.comicname + '</b> ' + str(self.db.db[self.db.currentcomic]['id']) + '\n ' + titl + '</span>')
+				self.label.set_markup(self.get_markup())
 				self.e_goweb.add(self.label)
 				self.e_goweb.show_all()
 
@@ -747,7 +852,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 				titl = str(self.db.db[self.db.currentcomic]['title'])
 				if len(titl) > 22:
 					titl = titl[:19] + "..."
-				self.label.set_markup('<span size="12000" face="monospace"> <b>' + self.comicname + '</b> ' + str(self.db.db[self.db.currentcomic]['id']) + '\n ' + titl + '</span>')
+				self.label.set_markup(self.get_markup())
 				self.e_goweb.add(self.label)
 				self.e_goweb.show_all()
 
@@ -771,7 +876,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 				titl = str(self.db.db[self.db.currentcomic]['title'])
 				if len(titl) > 22:
 					titl = titl[:19] + "..."
-				self.label.set_markup('<span size="12000" face="monospace"> <b>' + self.comicname + '</b> ' + str(self.db.db[self.db.currentcomic]['id']) + '\n ' + titl + '</span>')
+				self.label.set_markup(self.get_markup())
 				self.e_goweb.add(self.label)
 				self.e_goweb.show_all()
 				self.e_open.add(self.comic_image)
@@ -900,7 +1005,12 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 			return ret
 
 		
-		
+	def get_markup(self):
+		if len(self.comicname + " " + str(self.db.db[self.db.currentcomic]['id'])) < 20:
+			return '<span size="12000" face="monospace"> <b>' + self.comicname + '</b> ' + str(self.db.db[self.db.currentcomic]['id']) + '\n ' + str(self.db.db[self.db.currentcomic]['title']) + '</span>'
+		else:
+			return '<span size="12000" face="monospace"> <b>' + self.comicname + '</b>' + '\n ' + str(self.db.db[self.db.currentcomic]['title']) + '</span>'
+
 		
 		#check if settings file exists
 
@@ -953,40 +1063,64 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 		dialog.run()
 		dialog.destroy()
 
-	def show_comics(self, widget):
+	def show_comics(self, widget, data = None):
 		print "in comics dialog! yay!"
-		dialog = gtk.Dialog("Configure Search Engines", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
+		dialog = gtk.Dialog("Choose comics", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
 		comiclist = comics.keys()
 		comiclist.sort()
 		buttonlist = {}
 		self.connlist = {}
+		self.liststore = gtk.ListStore(str,str,bool)
 		for comicid in comiclist:
-			buttonlist[comicid] = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-			print buttonlist[comicid].get_name()
-			
-			print buttonlist[comicid].set_alignment(0, 0.5, 0, 0.5)	
 			active = False
 			for acomic in self.active_comics:
 				if acomic == comicid:
 					active = True
 					break
 			if active:
-				buttonlist[comicid].set_text(comics[comicid]['name'] + " *", comicid + ": click to remove")
-				self.connlist[comicid] = buttonlist[comicid].connect("clicked", self.remove_comic)
+				self.liststore.append(["* " + comics[comicid]['name'],comicid, active])
 			else:
-				buttonlist[comicid].set_text(comics[comicid]['name'], comicid + ": click to add")
-				self.connlist[comicid] = buttonlist[comicid].connect("clicked", self.add_comic)
-			dialog.vbox.pack_start(buttonlist[comicid], True, True, 0)
+				self.liststore.append(["  " + comics[comicid]['name'],comicid, active])
+		self.treeview = hildon.GtkTreeView(gtk.HILDON_UI_MODE_EDIT)
+		self.treeview.set_model(self.liststore)
+		self.treeview.set_reorderable(True)
+		tvcolumn = gtk.TreeViewColumn('Column 0')
+		self.treeview.append_column(tvcolumn)
+		cell = gtk.CellRendererText()
+		tvcolumn.pack_start(cell, True)
+		tvcolumn.add_attribute(cell, 'text', 0)
 
-		
+		ts = hildon.PannableArea()
+	
+		ts.set_size_request(-1, 280)
+		ts.add(self.treeview)
+		selection = self.treeview.get_selection()
+
+		dialog.vbox.pack_start(ts,True,True,0)
 		dialog.show_all()
+		selection.connect("changed", self.pick_comic, comiclist)
 		dialog.run()
 		dialog.destroy()
 
-	def add_comic(self, widget):
-		print "add " + widget.get_title()
-		splt = string.split(widget.get_value(), ':', 1)
-		comicid = splt[0]
+	def pick_comic(self, widget, data):
+		selected = widget.get_selected()
+		if selected == None:
+			return False
+		
+		print selected[0].get_value(selected[1],0) + ", " + selected[0].get_value(selected[1],1) + ", " + str(selected[0].get_value(selected[1],2))
+		if selected[0].get_value(selected[1],2):
+			self.remove_comic(selected[0].get_value(selected[1],1))
+			splt = selected[0].get_value(selected[1],0)[1:]
+			selected[0].set_value(selected[1],0," " + splt)
+			selected[0].set_value(selected[1],2,False)
+		else:
+			self.add_comic(selected[0].get_value(selected[1],1))
+			splt = selected[0].get_value(selected[1],0)[1:]
+			selected[0].set_value(selected[1],0,"*" + splt)
+			selected[0].set_value(selected[1],2,True)
+
+				
+	def add_comic(self, comicid):
 		if os.path.isfile(activecomics) == True:
 			print "added " + comicid + " to " + str(self.active_comics)
 			try:
@@ -997,16 +1131,8 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 			dbw = csv.writer(dbf)
 			dbw.writerow(self.active_comics)
 			dbf.close()
-		widget.set_text(comics[comicid]['name'] + " *", comicid + ": click to remove")
-		widget.disconnect(self.connlist[comicid])
-		self.connlist[comicid] = widget.connect("clicked", self.remove_comic)
 
-	def remove_comic(self, widget):
-		print "remove " + widget.get_title()
-		if len(self.active_comics) == 1:
-			return
-		splt = string.split(widget.get_value(), ':', 1)
-		comicid = splt[0]
+	def remove_comic(self, comicid):
 		if os.path.isfile(activecomics) == True:
 			print "removing " + comicid + " from " + str(self.active_comics)
 			try:
@@ -1017,9 +1143,7 @@ class ComicHomePlugin(hildondesktop.HomePluginItem):
 			dbw = csv.writer(dbf)
 			dbw.writerow(self.active_comics)
 			dbf.close()
-		widget.set_text(comics[comicid]['name'], comicid + ": click to add")
-		widget.disconnect(self.connlist[comicid])
-		self.connlist[comicid] = widget.connect("clicked", self.add_comic)
+
 
 	def get_size_settings(self):
 		# defaults = {'width':480,'height':230}
